/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.sys.startstopp.prozessvew;

import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.startstopp.skriptvew.GlobaleDaten;
import de.bsvrz.sys.startstopp.skriptvew.SkriptVerwaltung;
import de.bsvrz.sys.startstopp.skriptvew.StartStoppApp;
import de.bsvrz.sys.startstopp.skriptvew.StartStoppVerwaltung;
import de.bsvrz.sys.startstopp.skriptvew.XMLToolsTest;

/**
 * Klasse zum testen der Kommunikation mit dem DaV
 * @author Dambach Werke GmbH
 */
public class DaVKommunikationTest
{
  /**
   * Name einer temp. Datei 
   */
  private static String m_datei        = "prozessverwaltungTest.xml";
  
  /**
   * Datenverteilerverbindung
   */
  private static ClientDavInterface m_connection = null;

  /**
   * Skripverwaltung
   */
  private static SkriptVerwaltung m_sv = null;
  
  /**
   * Prozessverwaltung
   */
  private static ProzessVerwaltung m_pv;
  
  /**
   * Absender ID
   */
  private static int m_absender = 1234;

  /**
   * Instanz der Klasse DaVKommunikation
   */
  private DaVKommunikation m_dav = DaVKommunikation.getInstanz();
  
  /**
   * Merker, dass zu prfender Datensatz empfangen wurde
   */
  boolean m_empfangen = false;

  /**
   * Merker, dass die zu prfenden Namen gefunden wurden
   */
  private HashMap<String, Boolean> m_gefundenInProzessInfo;
  private HashMap<String, Boolean> m_gefundenInStartStoppInfo;

  /**
   * Merker, dass die Prozessinformationen empfangen wurden
   */
  private boolean m_startStoppInfoOk;

  /**
   * Merker, dass die StartStopp Informationen empfangen wurden
   */
  private boolean m_startStoppBlockOk;

  /**
   * Fehlertext
   */
  private String m_fehlerText = null;

  private boolean m_fehler;

  /** 
   * Wartezeit die aktiviert werden kann, um die Prozessinformationen zustzlich am GTM
   * auszuwerten
   */
  private long m_warteZeit = 0;  // in Millisekungen
  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    // Zugangsdaten DaV
    
    String rechner  = "localhost";
    int port        = 8083;
    String benutzer = "Tester";
    String password = "geheim";
    
    // Globale Einstellungen die fr die Klasse DaVKommunikation bentigt werden
    
    GlobaleDaten glob = GlobaleDaten.getInstanz();
    
    glob.setDavAdresse( rechner );
    glob.setDavPort( port );
    glob.setDavBenutzer( benutzer );
    glob.setDavPasswort( password );

    // Notwendige Initialisierungen
    
    String[] args = new String [2];
    
    args [0] = "-startStoppKonfiguration=C:/TestStartStopp"; 
    args [1] = "-_versionieren=C:/TestStartStopp/StartStopp.xml";
    
    StartStoppApp._applicationLabel = StartStoppApp.createApplicationLabel( args );
    ProzessVerwaltung.getInstanz().setKernSystemGestartet( true );
    
    // Datenverteilerverbindung aufbauen zum Prfen der gesendeten Datenstze
    
    ClientDavParameters parameters = new ClientDavParameters();
    
    parameters.setDavCommunicationAddress( rechner );
    parameters.setDavCommunicationSubAddress( port );
    parameters.setUserName( benutzer );
    parameters.setUserPassword( password );

    m_connection = new ClientDavConnection(parameters);
    m_connection.connect();
    m_connection.login();
    
    // Testumgebung initialisieren
    
    StartStoppApp a = new StartStoppApp ();
    
    a.setStartStoppKonfiguration( "C:\\JUnit");
    a.setStartStoppInkarnationsName( "StartStopp" );
    a.bestimmeRechnerAdresse();
    
    // Prozessverwaltung
    
    m_pv = ProzessVerwaltung.getInstanz();

  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
    m_connection.disconnect( false, "" );
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.DaVKommunikation#baueVerbindungDaVAuf()}.
   * Methode prft ob die verbindung zu einem laufenen Datenverteiler aufgebaut werden kann.
   */
  @Test
  public void testBaueVerbindungDaVAuf()
  {
    if (m_dav.istVerbunden())
      fail( "Bereits mit DaV verbunden" );

    if (!m_dav.baueVerbindungDaVAuf())
      fail( "Verbindungsaufbau nicht mglich" );
    
    if (!m_dav.istVerbunden())
      fail( "Falsches Ergebnis von isVerbunden" );
  }
  
  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.DaVKommunikation#sendeProzessInfo()}.
   * {@link de.bsvrz.sys.startstopp.prozessvew.DaVKommunikation#sendeStartStoppInfo()}.
   * Methode prft ob alle Prozessinformationen einer konkreten XML-Datei versendet werden.
   */
  @Test
  public void testSendeProzessInfoGefuellt()
  {
    List<String> erwarteteNamen = Arrays.asList ( "Betriebsmeldungsverwaltung",
                                                  "UeberpruefungSystem",
                                                  "Applikation1.1",
                                                  "Applikation1.2",
                                                  "Parametrierung",
                                                  "Applikation1.4",
                                                  "Applikation1.3",
                                                  "Konfiguration",
                                                  "Datenverteiler");

    
    m_gefundenInProzessInfo = new HashMap<String, Boolean> ();
    Iterator<String> it = erwarteteNamen.iterator();
    while (it.hasNext())
      m_gefundenInProzessInfo.put( it.next(), false );

    m_gefundenInStartStoppInfo = new HashMap<String, Boolean> ();
    it = erwarteteNamen.iterator();
    while (it.hasNext())
      m_gefundenInStartStoppInfo.put( it.next(), false );

    // Testdatei aus Resourcen anlegen und in Scriptverwaltung eintragen
    
    XMLToolsTest test = new XMLToolsTest ();
    
    m_pv.reset();
    StartStoppVerwaltung.getInstanz().reset();
    
    // Skriptverwaltung anlegen

    String xmlAlsString = test.einlesenAusResource( "../xml/startStopp.xml" );
    
    test.schreibeInDatei( m_datei, xmlAlsString );
    
    GlobaleDaten.getInstanz().removeAllListener();
    m_sv = new SkriptVerwaltung (m_datei, m_absender, 0);
    
    if (m_sv.isFehlerStartStopp())
      fail ( m_sv.getFehlerText() );

    // Verbindung zum DaV aufbauen und die Prozessinformationen versenden
    
    if (!m_dav.baueVerbindungDaVAuf())
      fail( "Verbindungsaufbau nicht mglich" );

    m_dav.anmeldeDaV();
  
    m_dav.sendeProzessInfo();
    m_dav.sendeStartStoppInfo();
    
    // Von JUNIT aus am DaV anmelden und die empfangene Prozessinformationen auswerten
    
    final Object sync = new Object();

    m_empfangen = false;
    
    final String _atg1 = "atg.prozessInfo";
    final String _atg2 = "atg.startStoppInfo";

    m_startStoppInfoOk = false;
    m_startStoppBlockOk = false;

    m_fehlerText = null;
    
    // Neuen Empfnger anlegen der auf die Attributgruppe wartet
    
    ClientReceiverInterface empf = new ClientReceiverInterface()
    {
      public void update( ResultData[] results )
      {
        for (int i = 0; i < results.length; i++) 
        {
          ResultData dat = results [i];

          // Auslesen der Attributgruppe

          if (dat.getDataDescription().getAttributeGroup().getPid().equals( _atg1 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              Array e = dat.getData().getArray( "Prozesse" );
              
              int anzahl = e.getLength();

              if (anzahl == 9)
              {
                for (int a = 0; a < anzahl; a++) 
                {
                  String name = e.getItem( a ).getTextValue( "Name" ).getValueText();
                  
                  if (m_gefundenInProzessInfo.containsKey( name ))
                    m_gefundenInProzessInfo.put( name, true );
                  else
                    m_fehlerText = _atg1 + " Unerwarteter Prozessname gefunden: " + name;
                }
                
                if (m_gefundenInProzessInfo.containsValue( false ))
                  m_fehlerText = _atg1 + " Nicht alle erwarteten Prozesse gefunden";
                
                if (m_fehlerText == null)
                  m_startStoppInfoOk = true;
                else
                {
                  synchronized (sync)
                  {
                    sync.notify();
                  }
                }
              }
            }
          }
          
          if (dat.getDataDescription().getAttributeGroup().getPid().equals( _atg2 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              Array e = dat.getData().getArray( "StartStoppBloecke" );
              
              int anzahl = e.getLength();

              if (anzahl == 1)
              {
                for (int a = 0; a < anzahl; a++) 
                {
                  String startStoppID = e.getItem( a ).getTextValue( "StartStoppID" ).getValueText();
                  Array e1 = e.getItem( a ).getArray( "Inkarnationen" );
                  
                  int anzahl1 = e1.getLength();
                  
                  if (anzahl1 == 9)
                  {
                    for (int a1=0; a1<anzahl1; ++a1)
                    {
                      String nameInkarnation = e1.getItem( a1 ).getTextValue( "ProzessID" ).getValueText();

                      if (!nameInkarnation.contains( startStoppID ))
                        m_fehlerText = _atg2 + " Fehler in Inkarnationsname: " + nameInkarnation;
                      
                      nameInkarnation = nameInkarnation.replace( startStoppID + "_", "" );
                      
                      if (m_gefundenInStartStoppInfo.containsKey( nameInkarnation ))
                        m_gefundenInStartStoppInfo.put( nameInkarnation, true );
                      else
                        m_fehlerText = _atg2 + " Unerwarteter Prozessname gefunden: " + nameInkarnation;
                    }

                    if (m_gefundenInStartStoppInfo.containsValue( false ))
                      m_fehlerText = _atg2 + " Nicht alle erwarteten Prozesse gefunden";

                    if (m_fehlerText == null)
                      m_startStoppBlockOk = true;
                    else
                    {
                      synchronized (sync)
                      {
                        sync.notify();
                      }
                    }
                  }
                }
              }
            }
          }
        }

        if (m_startStoppInfoOk && m_startStoppBlockOk)
        {
          m_empfangen = true;
          
          synchronized (sync)
          {
            sync.notify();
          }
        }
      }
    };

    // Anmelden zum Empfangen der zu prfenden Attributgruppe
    
    AttributeGroup atg1 = m_connection.getDataModel().getAttributeGroup( _atg1 );
    Aspect asp1         = m_connection.getDataModel().getAspect("asp.zustand");

    AttributeGroup atg2 = m_connection.getDataModel().getAttributeGroup( _atg2 );
    Aspect asp2         = m_connection.getDataModel().getAspect("asp.zustand");

    String pidRechner = m_dav.getPidRechner();
    SystemObject sysObj = m_connection.getDataModel().getObject(pidRechner);
    
    DataDescription dd1 = new DataDescription(atg1, asp1);
    DataDescription dd2 = new DataDescription(atg2, asp2);
  
    m_connection.subscribeReceiver(empf, sysObj, dd1, ReceiveOptions.normal(), ReceiverRole.receiver());
    m_connection.subscribeReceiver(empf, sysObj, dd2, ReceiveOptions.normal(), ReceiverRole.receiver());

    // Warten auf Antwort des ClientReceiverInterface
    
    synchronized (sync)
    {
      try
      {
        sync.wait(30000);
      }
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }
    }

    // Sleep wird eingebaut damit man die Prozessinformationen mit dem GTM sehen kann
    
    mySleep (m_warteZeit);
    
    // Verbindung zum DaV wieder trennen

    m_connection.unsubscribeReceiver( empf, sysObj, dd1);
    m_connection.unsubscribeReceiver( empf, sysObj, dd2);
    
    m_dav.allesAbmelden();
    
    mySleep( 2000 );
    
    m_dav.trenneDaV();
    
    if (!m_empfangen)
    {
      if (m_fehlerText != null)
        fail (m_fehlerText);
      else
      {
        if (!m_startStoppInfoOk)
          fail ("Attributgruppe " + _atg1 + " nicht korrekt empfangen");
        
        if (!m_startStoppBlockOk)
          fail ("Attributgruppe " + _atg2 + " nicht korrekt empfangen");
      }
    }
  }
  
  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.DaVKommunikation#sendeProzessInfo()}.
   * {@link de.bsvrz.sys.startstopp.prozessvew.DaVKommunikation#sendeStartStoppInfo()}.
   * Methode prft ob leere Prozessinformationen versendet werden.
   */
  @Test
  public void testSendeProzessInfoLeer()
  {
    m_pv.reset();
    StartStoppVerwaltung.getInstanz().reset();
    
    GlobaleDaten.getInstanz().removeAllListener();

    // Verbindung zum DaV aufbauen und die Prozessinformationen versenden
    
    if (!m_dav.baueVerbindungDaVAuf())
      fail( "Verbindungsaufbau nicht mglich" );

    m_dav.anmeldeDaV();
  
    m_dav.sendeProzessInfo();
    m_dav.sendeStartStoppInfo();
    
    // Von JUNIT aus am DaV anmelden und die empfangene Prozessinformationen auswerten
    
    final Object sync = new Object();

    m_empfangen = false;
    
    final String _atg1 = "atg.prozessInfo";
    final String _atg2 = "atg.startStoppInfo";

    m_startStoppInfoOk = false;
    m_startStoppBlockOk = false;

    // Neuen Empfnger anlegen der auf die Attributgruppe wartet
    
    ClientReceiverInterface empf = new ClientReceiverInterface()
    {
      public void update( ResultData[] results )
      {
        for (int i = 0; i < results.length; i++) 
        {
          ResultData dat = results [i];

          // Auslesen der Attributgruppe

          if (dat.getDataDescription().getAttributeGroup().getPid().equals( _atg1 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              Array e = dat.getData().getArray( "Prozesse" );
              
              System.out.println("Prozesse " + e.getLength());
              int anzahl = e.getLength();

              if (anzahl == 0)
                m_startStoppInfoOk = true;
            }
          }
          
          if (dat.getDataDescription().getAttributeGroup().getPid().equals( _atg2 ))
          {
            // Prfen ob Daten vorhanden sind
            
            if (dat.hasData())
            {
              Array e = dat.getData().getArray( "StartStoppBloecke" );
              
              System.out.println("SSB " + e.getLength());
              int anzahl = e.getLength();

              if (anzahl == 0)
                m_startStoppBlockOk = true;
            }
          }
        }

        if (m_startStoppInfoOk && m_startStoppBlockOk)
        {
          m_empfangen = true;
          
          synchronized (sync)
          {
            sync.notify();
          }
        }
      }
    };

    // Anmelden zum Empfangen der zu prfenden Attributgruppe
    
    AttributeGroup atg1 = m_connection.getDataModel().getAttributeGroup( _atg1 );
    Aspect asp1         = m_connection.getDataModel().getAspect("asp.zustand");

    AttributeGroup atg2 = m_connection.getDataModel().getAttributeGroup( _atg2 );
    Aspect asp2         = m_connection.getDataModel().getAspect("asp.zustand");

    String pidRechner = m_dav.getPidRechner();
    SystemObject sysObj = m_connection.getDataModel().getObject(pidRechner);
    
    DataDescription dd1 = new DataDescription(atg1, asp1);
    DataDescription dd2 = new DataDescription(atg2, asp2);
  
    m_connection.subscribeReceiver(empf, sysObj, dd1, ReceiveOptions.normal(), ReceiverRole.receiver());
    m_connection.subscribeReceiver(empf, sysObj, dd2, ReceiveOptions.normal(), ReceiverRole.receiver());

    // Warten auf Antwort des ClientReceiverInterface
    
    synchronized (sync)
    {
      try
      {
        sync.wait(30000);
      }
      catch (InterruptedException e)
      {
        e.printStackTrace();
      }
    }

    // Sleep wird eingebaut damit man die Prozessinformationen mit dem GTM sehen kann
    
    mySleep (m_warteZeit);

    // Verbindung zum DaV wieder trennen

    m_connection.unsubscribeReceiver( empf, sysObj, dd1);
    m_connection.unsubscribeReceiver( empf, sysObj, dd2);

    m_dav.allesAbmelden();
    
    mySleep( 2000 );
    m_dav.trenneDaV();

    if (!m_empfangen)
    {
      if (!m_startStoppInfoOk)
        fail ("Attributgruppe " + _atg1 + " nicht korrekt empfangen");
      
      if (!m_startStoppBlockOk)
        fail ("Attributgruppe " + _atg2 + " nicht korrekt empfangen");
    }
  }
  
  /**
   * Sleep
   * @param ms Millisekunden
   */
  private void mySleep (long ms)
  {
    try
    {
      Thread.sleep (ms);
    }
    catch (InterruptedException e)
    {
      e.printStackTrace();
    } 
  }
}


