/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.sys.startstopp.prozessvew;

import static org.junit.Assert.*;

import java.io.File;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 * Klasse zum Testen der Klasse SchreibeProtokoll
 * @author Dambach Werke GmbH
 */
public class SchreibeProtokollTest
{

  /**
   * @throws java.lang.Exception
   */
  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @Before
  public void setUp() throws Exception
  {
  }

  /**
   * @throws java.lang.Exception
   */
  @After
  public void tearDown() throws Exception
  {
  }

  /**
   * Test Methode for {@link de.bsvrz.sys.startstopp.prozessvew.SchreibeProtokoll#schreibeSynchronisiertInDatei(java.lang.String, java.lang.String, long)}.
   * Prfung erfolgt indem solange in die Dateien geschrieben wird, bis ein Dateiwechsel festgestellt wird. Danach
   * wird die Gre der bisherigen Datei geprft. Der ganze Vorgang wird mehrfach wiederholt.
   */
  @Test
  public void testSchreibeSynchronisiertInDatei()
  {
    boolean ok = true;
    
    String dateiName = "C:\\Junit\\TestSchreibeSynchronisiertProtkoll.txt";
    
    long maxGroesseInMB   = 1;  // in Megabyte
    long maxGroesseInByte = maxGroesseInMB * 1048576;
    
    SchreibeProtokoll sp = SchreibeProtokoll.getInstanz();

    // Teststring erzeugen
    
    String inhalt = "";
    for (int j=0; j<1000; ++j) 
      inhalt += "a";
    inhalt += "\n";

    File aktDatei = null;
    
    int anzWechsel = 0;
    int zuPrfendeWechsel = 10;

    while ((anzWechsel < zuPrfendeWechsel) && ok)
    {
      sp.schreibeSynchronisiertInDatei( dateiName, inhalt, maxGroesseInMB );
      
      File f = sp.getAktDatei();
      
      if (f == null)
        ok = false;
      else
      {
        if (aktDatei == null)
          aktDatei = f;
        else
        {
          if (!f.getAbsoluteFile().equals(aktDatei.getAbsoluteFile()))
          {
            long groesse = aktDatei.length();
            if ((groesse < maxGroesseInByte) || (groesse > maxGroesseInByte + inhalt.length()))
            {
              fail ("Falsche Grsse: " + groesse);
              ok = false;
            }
              
            aktDatei = f;
            anzWechsel++;
          }
        }
      }
    }

    assertEquals( true, ok) ;
  }

  /**
   * Test Methode fr {@link de.bsvrz.sys.startstopp.prozessvew.SchreibeProtokoll#schreibeUnSynchronisiertInDatei(java.lang.String, java.lang.String, long)}.
   */
  @Test
  public void testSchreibeUnSynchronisiertInDatei()
  {
    boolean ok = true;
    
    String dateiName = "C:\\Junit\\TestSchreibeUnsynchronisiertProtkoll.txt";
    
    long maxGroesseInMB   = 1;  // in Megabyte
    long maxGroesseInByte = maxGroesseInMB * 1048576;
    
    SchreibeProtokoll sp = SchreibeProtokoll.getInstanz();

    // Teststring erzeugen
    
    String inhalt = "";
    for (int j=0; j<1000; ++j) 
      inhalt += "a";
    inhalt += "\n";

    File aktDatei = null;
    
    int anzWechsel = 0;
    int zuPrfendeWechsel = 10;

    while ((anzWechsel < zuPrfendeWechsel) && ok)
    {
      sp.schreibeUnSynchronisiertInDatei( dateiName, inhalt, maxGroesseInMB );
      
      File f = sp.getAktDatei();
      
      if (f == null)
        ok = false;
      else
      {
        if (aktDatei == null)
          aktDatei = f;
        else
        {
          if (!f.getAbsoluteFile().equals(aktDatei.getAbsoluteFile()))
          {
            long groesse = aktDatei.length();
            if ((groesse < maxGroesseInByte) || (groesse > maxGroesseInByte + inhalt.length()))
            {
              fail ("Falsche Grsse: " + groesse);
              ok = false;
            }
              
            aktDatei = f;
            anzWechsel++;
          }
        }
      }
    }

    assertEquals( true, ok) ;
  }
}
